//------------------------------------------------------------------
#property copyright "mladen"
#property link      "mladenfx@gmail.com"
//------------------------------------------------------------------
#property indicator_separate_window
#property indicator_buffers 4
#property indicator_color1  clrDodgerBlue
#property indicator_color2  clrSandyBrown
#property indicator_color3  clrSandyBrown
#property indicator_color4  clrSilver
#property indicator_width1  2
#property indicator_width2  2
#property indicator_width3  2
#property strict

//
//
//
//
//

enum colorOn
{
   clrOnSlope, //Color change on slope change
   clrOnZero,  // Color change on zero cross
   clrOnSign   // Color change on signal cross
};
extern int    FastMAPeriod = 23;  // Fast macd period
extern int    SlowMAPeriod = 50;  // Slow macd period
extern int    SignalPeriod = 9;   // Signal period
extern int    StdPeriod    = 50;  // Standard deviation used for normalization
extern ENUM_APPLIED_PRICE Price         = PRICE_CLOSE; // Price to use 
extern colorOn            ColorChangeOn = clrOnSlope;  // Color change on : 

//
//
//
//
//

double macdBuffer[];
double macdBufferUA[];
double macdBufferUB[];
double signal[];
double slope[];
double prices[];


//------------------------------------------------------------------
//
//------------------------------------------------------------------
//
//
//
//
//

int init()
{
   IndicatorBuffers(6);
      SetIndexBuffer(0,macdBuffer);
      SetIndexBuffer(1,macdBufferUA);
      SetIndexBuffer(2,macdBufferUB);
      SetIndexBuffer(3,signal);
      SetIndexBuffer(4,slope);
      SetIndexBuffer(5,prices);
   IndicatorShortName("MACD (std normalized) ("+(string)FastMAPeriod+","+(string)SlowMAPeriod+")");
   return(0);
}

int deinit()
{
   return(0);
}

//------------------------------------------------------------------
//
//------------------------------------------------------------------
//
//
//
//
//

int start()
{
   int counted_bars=IndicatorCounted();
      if(counted_bars < 0) return(-1);
      if(counted_bars>0) counted_bars--;
         int limit = MathMin(Bars-counted_bars,Bars-1);

   //
   //
   //
   //
   //
   
   if (slope[limit] == 1) CleanPoint(limit,macdBufferUA,macdBufferUB);
   for(int i = limit; i >= 0; i--)
   {
      prices[i] = iMA(NULL,0,1,0,MODE_SMA,Price,i);
      double std = iStdDev(NULL,0,StdPeriod,0,MODE_SMA,Price,i);
         if (std!=0)
            macdBuffer[i] = (iEma(prices[i],FastMAPeriod,i,0)-iEma(prices[i],SlowMAPeriod,i,1))/std;
            signal[i]     =  iEma(macdBuffer[i],SignalPeriod,i,2);
      
      if (i>=Bars-1) continue;
      
      //
      //
      //
      //
      //
      
      macdBufferUA[i] = EMPTY_VALUE;
      macdBufferUB[i] = EMPTY_VALUE;
      slope[i]       = slope[i+1];
      switch (ColorChangeOn)
      {
         case clrOnSlope: 
            if (macdBuffer[i] > macdBuffer[i+1]) slope[i] =  1;
            if (macdBuffer[i] < macdBuffer[i+1]) slope[i] = -1;
            break;
         case clrOnZero: 
            if (macdBuffer[i] > 0) slope[i] =  1;
            if (macdBuffer[i] < 0) slope[i] = -1;
            break;
         default : 
            if (macdBuffer[i] > signal[i]) slope[i] =  1;
            if (macdBuffer[i] < signal[i]) slope[i] = -1;
            break;
      }      
      if (slope[i]==-1) PlotPoint(i,macdBufferUA,macdBufferUB,macdBuffer);
   }   
   return(0);
}


//-------------------------------------------------------------------
//                                                                  
//-------------------------------------------------------------------
//
//
//
//
//

double workEma[][3];
double iEma(double price, double period, int r, int instanceNo=0)
{
   if (ArrayRange(workEma,0)!= Bars) ArrayResize(workEma,Bars); r=Bars-r-1;
   if (period<=1) { workEma[r][instanceNo]=price; return(price); }

   //
   //
   //
   //
   //
      
   workEma[r][instanceNo] = price;
   double alpha = 2.0 / (1.0+period);
   if (r>0)
          workEma[r][instanceNo] = workEma[r-1][instanceNo]+alpha*(price-workEma[r-1][instanceNo]);
   return(workEma[r][instanceNo]);
}

//-------------------------------------------------------------------
//                                                                  
//-------------------------------------------------------------------
//
//
//
//
//

void CleanPoint(int i,double& first[],double& second[])
{
   if (i>=Bars-3) return;
   if ((second[i]  != EMPTY_VALUE) && (second[i+1] != EMPTY_VALUE))
        second[i+1] = EMPTY_VALUE;
   else
      if ((first[i] != EMPTY_VALUE) && (first[i+1] != EMPTY_VALUE) && (first[i+2] == EMPTY_VALUE))
          first[i+1] = EMPTY_VALUE;
}

void PlotPoint(int i,double& first[],double& second[],double& from[])
{
   if (i>=Bars-2) return;
   if (first[i+1] == EMPTY_VALUE)
      if (first[i+2] == EMPTY_VALUE) 
            { first[i]  = from[i];  first[i+1]  = from[i+1]; second[i] = EMPTY_VALUE; }
      else  { second[i] =  from[i]; second[i+1] = from[i+1]; first[i]  = EMPTY_VALUE; }
   else     { first[i]  = from[i];                           second[i] = EMPTY_VALUE; }
}